@@ -1,35 +1,58 @@
 
-#' Creates attention layer
+
+#' Dot-product attention layer, a.k.a. Luong-style attention
 #'
-#' Dot-product attention layer, a.k.a. Luong-style attention.
+#' @details
+#' inputs are `query` tensor of shape `[batch_size, Tq, dim]`, `value` tensor
+#' of shape `[batch_size, Tv, dim]` and `key` tensor of shape
+#' `[batch_size, Tv, dim]`. The calculation follows the steps:
 #'
-#' @inheritParams layer_dense
+#' 1. Calculate scores with shape `[batch_size, Tq, Tv]` as a `query`-`key` dot
+#'    product: `scores = tf$matmul(query, key, transpose_b=TRUE)`.
+#' 2. Use scores to calculate a distribution with shape
+#'    `[batch_size, Tq, Tv]`: `distribution = tf$nn$softmax(scores)`.
+#' 3. Use `distribution` to create a linear combination of `value` with
+#'    shape `[batch_size, Tq, dim]`:
+#'    return `tf$matmul(distribution, value)`.
 #'
-#' @param inputs a list of inputs first should be the query tensor, the second the value tensor
-#' @param use_scale If True, will create a scalar variable to scale the attention scores.
-#' @param causal Boolean. Set to True for decoder self-attention. Adds a mask such that position i cannot attend to positions j > i.
-#' This prevents the flow of information from the future towards the past.
+#'
+#' @param inputs List of the following tensors:
+#'
+#'   -  query: Query Tensor of shape `[batch_size, Tq, dim]`.
+#'
+#'   -  value: Value Tensor of shape `[batch_size, Tv, dim]`.
+#'
+#'   -  key: Optional key Tensor of shape `[batch_size, Tv, dim]`. If not
+#'   given, will use value for both key and value, which is the most common
+#'   case.
+#'
+#' @param use_scale If `TRUE`, will create a scalar variable to scale the attention
+#' scores.
+#'
+#' @param dropout Float between 0 and 1. Fraction of the units to drop for the
+#' attention scores. Defaults to 0.0.
+#'
+#' @param score_mode Function to use to compute attention scores, one of
+#' `{"dot", "concat"}`. `"dot"` refers to the dot product between the query
+#' and key vectors. `"concat"` refers to the hyperbolic tangent of the
+#' concatenation of the query and key vectors.
+#'
+#' @param ... standard layer arguments (e.g., batch_size, dtype, name, trainable, weights)
 #'
 #' @family core layers
 #' @family attention layers
 #'
+#'
+#' @seealso
+#'   +  <https://www.tensorflow.org/api_docs/python/tf/keras/layers/Attention>
 #' @export
-layer_attention <- function(inputs,use_scale=FALSE, causal = FALSE, batch_size = NULL, dtype = NULL,
-                                name = NULL, trainable = NULL, weights = NULL) {
-  if (!is_tensorflow_implementation() || !tensorflow::tf_version() >= "1.14")
-    stop("layer_dense_features requires TensorFlow implementation and version >= 1.14")
-  create_layer(keras$layers$Attention, inputs, list(
-      use_scale = use_scale,
-      causal = causal,
-      batch_size = batch_size,
-      dtype = dtype,
-      name = name,
-      trainable = trainable,
-      weights = weights)
-    )
-
-
-}
+layer_attention <-
+  function(inputs, use_scale = FALSE, score_mode = "dot", ..., dropout = NULL)
+  {
+    args <- capture_args(match.call(), ignore = "inputs")
+    args$dropout <- dropout
+    create_layer(keras$layers$Attention, inputs, args)
+  }
 
 #' MultiHeadAttention layer
 #'
