@@ -782,6 +782,27 @@ function(object, num_tokens=NULL, output_mode = "multi_hot", sparse = FALSE, ...
 #' case, the FarmHash64 hash function is used). It also supports
 #' list of 2 unsigned integer numbers, see reference paper for details.
 #'
+#' @param output_mode Specification for the output of the layer. Defaults to
+#' `"int"`.  Values can be `"int"`, `"one_hot"`, `"multi_hot"`, or
+#' `"count"` configuring the layer as follows:
+#'   - `"int"`: Return the integer bin indices directly.
+#'   - `"one_hot"`: Encodes each individual element in the input into an
+#'     array the same size as `num_bins`, containing a 1 at the input's bin
+#'     index. If the last dimension is size 1, will encode on that
+#'     dimension.  If the last dimension is not size 1, will append a new
+#'     dimension for the encoded output.
+#'   - `"multi_hot"`: Encodes each sample in the input into a single array
+#'     the same size as `num_bins`, containing a 1 for each bin index
+#'     index present in the sample. Treats the last dimension as the sample
+#'     dimension, if input shape is `(..., sample_length)`, output shape
+#'     will be `(..., num_tokens)`.
+#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of
+#'     the number of times the bin index appeared in the sample.
+#'
+#' @param sparse Boolean. Only applicable to `"one_hot"`, `"multi_hot"`,
+#' and `"count"` output modes. If `TRUE`, returns a `SparseTensor` instead of
+#' a dense `Tensor`. Defaults to `FALSE`.
+#'
 #' @param ... standard layer arguments.
 #'
 #' @family categorical features preprocessing layers
@@ -792,12 +813,14 @@ function(object, num_tokens=NULL, output_mode = "multi_hot", sparse = FALSE, ...
 #'   -  <https://keras.io/api/layers/preprocessing_layers/categorical/hashing/>
 #' @export
 layer_hashing <-
-function(object, num_bins, mask_value = NULL, salt = NULL, ...)
+function(object, num_bins, mask_value = NULL, salt = NULL,
+         output_mode = "int", sparse = FALSE, ...)
 {
   require_tf_version("2.6", "layer_hashing()")
   args <- capture_args(match.call(),
-                       list(num_bins = as.integer,
-                            salt = as_nullable_integer),
+                       c(standard_layer_arg_modifiers,
+                         num_bins = as.integer,
+                         salt = as_nullable_integer),
                        ignore = "object")
   create_layer(keras$layers$Hashing, object, args)
 }
@@ -807,42 +830,50 @@ function(object, num_bins, mask_value = NULL, salt = NULL, ...)
 #' A preprocessing layer which maps integer features to contiguous ranges.
 #'
 #' @details
-#' This layer maps a set of arbitrary integer input tokens into indexed
-#' integer output via a table-based vocabulary lookup. The layer's output indices
-#' will be contiguously arranged up to the maximum vocab size, even if the input
+#' This layer maps a set of arbitrary integer input tokens into indexed integer
+#' output via a table-based vocabulary lookup. The layer's output indices will
+#' be contiguously arranged up to the maximum vocab size, even if the input
 #' tokens are non-continguous or unbounded. The layer supports multiple options
 #' for encoding the output via `output_mode`, and has optional support for
 #' out-of-vocabulary (OOV) tokens and masking.
 #'
-#' The vocabulary for the layer can be supplied on construction or learned via
-#' `adapt()`. During `adapt()`, the layer will analyze a data set, determine the
-#' frequency of individual integer tokens, and create a vocabulary from them. If
-#' the vocabulary is capped in size, the most frequent tokens will be used to
-#' create the vocabulary and all others will be treated as OOV.
-#'
-#' There are two possible output modes for the layer.
-#' When `output_mode` is `"int"`,
-#' input integers are converted to their index in the vocabulary (an integer).
-#' When `output_mode` is `"multi_hot"`, `"count"`, or `"tf_idf"`, input integers
-#' are encoded into an array where each dimension corresponds to an element in
-#' the vocabulary.
-#'
 #' The vocabulary for the layer must be either supplied on construction or
 #' learned via `adapt()`. During `adapt()`, the layer will analyze a data set,
-#' determine the frequency of individual integer tokens, and create a vocabulary
-#' from them. If the vocabulary is capped in size, the most frequent tokens will
-#' be used to create the vocabulary and all others will be treated as OOV.
+#' determine the frequency of individual integer tokens, and create a
+#' vocabulary from them. If the vocabulary is capped in size, the most frequent
+#' tokens will be used to create the vocabulary and all others will be treated
+#' as OOV.
+#'
+#' There are two possible output modes for the layer.  When `output_mode` is
+#' `"int"`, input integers are converted to their index in the vocabulary (an
+#' integer).  When `output_mode` is `"multi_hot"`, `"count"`, or `"tf_idf"`,
+#' input integers are encoded into an array where each dimension corresponds to
+#' an element in the vocabulary.
+#'
+#' The vocabulary can optionally contain a mask token as well as an OOV token
+#' (which can optionally occupy multiple indices in the vocabulary, as set
+#' by `num_oov_indices`).
+#' The position of these tokens in the vocabulary is fixed. When `output_mode`
+#' is `"int"`, the vocabulary will begin with the mask token at index `0`,
+#' followed by OOV indices, followed by the rest of the vocabulary. When
+#' `output_mode` is `"multi_hot"`, `"count"`, or `"tf_idf"` the vocabulary will
+#' begin with OOV indices and instances of the mask token will be dropped.
+#'
+#' For an overview and full list of preprocessing layers, see the preprocessing
+#' [guide](https://www.tensorflow.org/guide/keras/preprocessing_layers).
 #'
 #' @inheritParams layer_dense
 #'
-#' @param max_tokens The maximum size of the vocabulary for this layer. If `NULL`,
-#' there is no cap on the size of the vocabulary. Note that this size
-#' includes the OOV and mask tokens. Default to `NULL.`
+#' @param max_tokens Maximum size of the vocabulary for this layer. This should
+#' only be specified when adapting the vocabulary or when setting
+#' `pad_to_max_tokens = TRUE`. If `NULL`, there is no cap on the size of the
+#' vocabulary. Note that this size includes the OOV and mask tokens.
+#' Defaults to `NULL.`
 #'
 #' @param num_oov_indices The number of out-of-vocabulary tokens to use. If this
 #' value is more than 1, OOV inputs are modulated to determine their OOV
-#' value. If this value is 0, OOV inputs will cause an error when calling the
-#' layer. Defaults to 1.
+#' value. If this value is 0, OOV inputs will cause an error when calling
+#' the layer. Defaults to 1.
 #'
 #' @param mask_token An integer token that represents masked inputs. When
 #' `output_mode` is `"int"`, the token is included in vocabulary and mapped
@@ -850,49 +881,60 @@ function(object, num_bins, mask_value = NULL, salt = NULL, ...)
 #' vocabulary and instances of the mask token in the input will be dropped.
 #' If set to `NULL`, no mask term will be added. Defaults to `NULL`.
 #'
-#' @param oov_token Only used when `invert` is `TRUE.` The token to return for OOV
+#' @param oov_token Only used when `invert` is `TRUE`. The token to return for OOV
 #' indices. Defaults to -1.
 #'
-#' @param vocabulary Optional. Either an array of integers or a string path to a text
-#' file. If passing an array, can pass a list, list, 1D numpy array, or 1D
-#' tensor containing the integer vocabulary terms. If passing a file path, the
-#' file should contain one line per term in the vocabulary. If this argument
-#' is set, there is no need to `adapt` the layer.
+#' @param vocabulary Optional. Either an array of integers or a string path to a
+#' text file. If passing an array, can pass a list, list, 1D numpy array,
+#' or 1D tensor containing the integer vocabulary terms. If passing a file
+#' path, the file should contain one line per term in the vocabulary. If
+#' this argument is set, there is no need to `adapt()` the layer.
 #'
-#' @param invert Only valid when `output_mode` is `"int"`. If `TRUE`, this layer will
+#' @param vocabulary_dtype The dtype of the vocabulary terms, for example
+#' `"int64"` or `"int32"`. Defaults to `"int64"`.
+#'
+#' @param idf_weights Only valid when `output_mode` is `"tf_idf"`. A list, list,
+#' 1D numpy array, or 1D tensor or the same length as the vocabulary,
+#' containing the floating point inverse document frequency weights, which
+#' will be multiplied by per sample term counts for the final `tf_idf`
+#' weight. If the `vocabulary` argument is set, and `output_mode` is
+#' `"tf_idf"`, this argument must be supplied.
+#'
+#' @param invert Only valid when `output_mode` is `"int"`. If TRUE, this layer will
 #' map indices to vocabulary items instead of mapping vocabulary items to
-#' indices. Default to `FALSE`.
+#' indices. Default to FALSE.
 #'
-#' @param output_mode Specification for the output of the layer. Defaults to `"int"`.
-#' Values can be `"int"`, `"one_hot"`, `"multi_hot"`, `"count"`, or
-#' `"tf_idf"` configuring the layer as follows:
+#' @param output_mode Specification for the output of the layer. Defaults to
+#' `"int"`.  Values can be `"int"`, `"one_hot"`, `"multi_hot"`, `"count"`,
+#' or `"tf_idf"` configuring the layer as follows:
 #'   - `"int"`: Return the vocabulary indices of the input tokens.
 #'   - `"one_hot"`: Encodes each individual element in the input into an
 #'     array the same size as the vocabulary, containing a 1 at the element
-#'     index. If the last dimension is size 1, will encode on that dimension.
-#'     If the last dimension is not size 1, will append a new dimension for
-#'     the encoded output.
+#'     index. If the last dimension is size 1, will encode on that
+#'     dimension.  If the last dimension is not size 1, will append a new
+#'     dimension for the encoded output.
 #'   - `"multi_hot"`: Encodes each sample in the input into a single array
 #'     the same size as the vocabulary, containing a 1 for each vocabulary
 #'     term present in the sample. Treats the last dimension as the sample
 #'     dimension, if input shape is (..., sample_length), output shape will
 #'     be (..., num_tokens).
-#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of the
-#'     number of times the token at that index appeared in the sample.
+#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of
+#'     the number of times the token at that index appeared in the sample.
 #'   - `"tf_idf"`: As `"multi_hot"`, but the TF-IDF algorithm is applied to
 #'     find the value in each token slot.
 #' For `"int"` output, any shape of input and output is supported. For all
 #' other output modes, currently only output up to rank 2 is supported.
 #'
 #' @param pad_to_max_tokens Only applicable when `output_mode` is `"multi_hot"`,
-#' `"count"`, or `"tf_idf"`. If TRUE, the output will have its feature axis
+#' `"count"`, or `"tf_idf"`. If `TRUE`, the output will have its feature axis
 #' padded to `max_tokens` even if the number of unique tokens in the
 #' vocabulary is less than max_tokens, resulting in a tensor of shape
-#' `[batch_size, max_tokens]` regardless of vocabulary size. Defaults to `FALSE`.
+#' `[batch_size, max_tokens]` regardless of vocabulary size. Defaults to
+#' FALSE.
 #'
 #' @param sparse Boolean. Only applicable when `output_mode` is `"multi_hot"`,
 #' `"count"`, or `"tf_idf"`. If `TRUE`, returns a `SparseTensor` instead of a
-#' dense `Tensor`. Defaults to `FALSE`.
+#' dense `Tensor`. Defaults to FALSE.
 #'
 #' @param ... standard layer arguments.
 #'
@@ -911,6 +953,8 @@ function(object,
          mask_token = NULL,
          oov_token = -1L,
          vocabulary = NULL,
+         vocabulary_dtype = "int64",
+         idf_weights = NULL,
          invert = FALSE,
          output_mode = 'int',
          sparse = FALSE,
@@ -919,9 +963,10 @@ function(object,
 {
   require_tf_version("2.6", "layer_integer_lookup()")
   args <- capture_args(match.call(),
-                       list(num_oov_indices = as.integer,
-                            oov_token = as.integer,
-                            output_mode = fix_string),
+                       c(standard_layer_arg_modifiers,
+                         num_oov_indices = as.integer,
+                         oov_token = as.integer,
+                         output_mode = fix_string),
                        ignore = "object")
   create_layer(keras$layers$IntegerLookup, object, args)
 }
@@ -932,14 +977,16 @@ function(object,
 #'
 #' @details
 #' This layer translates a set of arbitrary strings into integer output via a
-#' table-based vocabulary lookup.
+#' table-based vocabulary lookup. This layer will perform no splitting or
+#' transformation of input strings. For a layer than can split and tokenize
+#' natural language, see the `layer_text_vectorization()` layer.
 #'
 #' The vocabulary for the layer must be either supplied on construction or
 #' learned via `adapt()`. During `adapt()`, the layer will analyze a data set,
-#' determine the frequency of individual strings tokens, and create a vocabulary
-#' from them. If the vocabulary is capped in size, the most frequent tokens will
-#' be used to create the vocabulary and all others will be treated as
-#' out-of-vocabulary (OOV).
+#' determine the frequency of individual strings tokens, and create a
+#' vocabulary from them. If the vocabulary is capped in size, the most frequent
+#' tokens will be used to create the vocabulary and all others will be treated
+#' as out-of-vocabulary (OOV).
 #'
 #' There are two possible output modes for the layer.
 #' When `output_mode` is `"int"`,
@@ -951,60 +998,70 @@ function(object,
 #' The vocabulary can optionally contain a mask token as well as an OOV token
 #' (which can optionally occupy multiple indices in the vocabulary, as set
 #' by `num_oov_indices`).
-#' The position of these tokens in the vocabulary is fixed. When `output_mode` is
-#' `"int"`, the vocabulary will begin with the mask token (if set), followed by
-#' OOV indices, followed by the rest of the vocabulary. When `output_mode` is
-#' `"multi_hot"`, `"count"`, or `"tf_idf"` the vocabulary will begin with OOV
-#' indices and instances of the mask token will be dropped.
+#' The position of these tokens in the vocabulary is fixed. When `output_mode`
+#' is `"int"`, the vocabulary will begin with the mask token (if set), followed
+#' by OOV indices, followed by the rest of the vocabulary. When `output_mode`
+#' is `"multi_hot"`, `"count"`, or `"tf_idf"` the vocabulary will begin with
+#' OOV indices and instances of the mask token will be dropped.
+#'
+#' For an overview and full list of preprocessing layers, see the preprocessing
+#' [guide](https://www.tensorflow.org/guide/keras/preprocessing_layers).
 #'
 #' @inheritParams layer_dense
 #'
-#' @param max_tokens The maximum size of the vocabulary for this layer. If `NULL`,
-#' there is no cap on the size of the vocabulary. Note that this size
-#' includes the OOV and mask tokens. Default to `NULL.`
+#' @param max_tokens Maximum size of the vocabulary for this layer. This should
+#' only be specified when adapting the vocabulary or when setting
+#' `pad_to_max_tokens = TRUE`. If NULL, there is no cap on the size of the
+#' vocabulary. Note that this size includes the OOV and mask tokens.
+#' Defaults to NULL.
 #'
 #' @param num_oov_indices The number of out-of-vocabulary tokens to use. If this
-#' value is more than 1, OOV inputs are hashed to determine their OOV value.
-#' If this value is 0, OOV inputs will cause an error when calling the layer.
-#' Defaults to 1.
+#' value is more than 1, OOV inputs are hashed to determine their OOV
+#' value. If this value is 0, OOV inputs will cause an error when calling
+#' the layer. Defaults to 1.
 #'
 #' @param mask_token A token that represents masked inputs. When `output_mode` is
 #' `"int"`, the token is included in vocabulary and mapped to index 0. In
 #' other output modes, the token will not appear in the vocabulary and
-#' instances of the mask token in the input will be dropped. If set to `NULL`,
-#' no mask term will be added. Defaults to `NULL`.
+#' instances of the mask token in the input will be dropped. If set to
+#' NULL, no mask term will be added. Defaults to `NULL`.
 #'
-#' @param oov_token Only used when `invert` is TRUE. The token to return for OOV
+#' @param oov_token Only used when `invert` is `TRUE`. The token to return for OOV
 #' indices. Defaults to `"[UNK]"`.
 #'
-#' @param vocabulary Optional. Either an array of strings or a string path to a text
-#' file. If passing an array, can pass a list, list, 1D numpy array, or 1D
-#' tensor containing the string vocabulary terms. If passing a file path, the
-#' file should contain one line per term in the vocabulary. If this argument
-#' is set, there is no need to `adapt` the layer.
+#' @param vocabulary Optional. Either an array of strings or a string path to a
+#' text file. If passing an array, can pass a character vector or
+#' or 1D tensor containing the string vocabulary terms. If passing a file
+#' path, the file should contain one line per term in the vocabulary. If
+#' this argument is set, there is no need to `adapt()` the layer.
 #'
-#' @param encoding String encoding. Default of `NULL` is equivalent to `"utf-8"`.
+#' @param idf_weights Only valid when `output_mode` is `"tf_idf"`.
+#' An array, or 1D tensor or the same length as the vocabulary,
+#' containing the floating point inverse document frequency weights, which
+#' will be multiplied by per sample term counts for the final `tf_idf`
+#' weight. If the `vocabulary` argument is set, and `output_mode` is
+#' `"tf_idf"`, this argument must be supplied.
 #'
-#' @param invert Only valid when `output_mode` is `"int"`. If TRUE, this layer will
+#' @param invert Only valid when `output_mode` is `"int"`. If `TRUE`, this layer will
 #' map indices to vocabulary items instead of mapping vocabulary items to
 #' indices. Default to `FALSE`.
 #'
-#' @param output_mode Specification for the output of the layer. Defaults to `"int"`.
-#' Values can be `"int"`, `"one_hot"`, `"multi_hot"`, `"count"`, or
-#' `"tf_idf"` configuring the layer as follows:
+#' @param output_mode Specification for the output of the layer. Defaults to
+#' `"int"`.  Values can be `"int"`, `"one_hot"`, `"multi_hot"`, `"count"`,
+#' or `"tf_idf"` configuring the layer as follows:
 #'   - `"int"`: Return the raw integer indices of the input tokens.
 #'   - `"one_hot"`: Encodes each individual element in the input into an
 #'     array the same size as the vocabulary, containing a 1 at the element
-#'     index. If the last dimension is size 1, will encode on that dimension.
-#'     If the last dimension is not size 1, will append a new dimension for
-#'     the encoded output.
+#'     index. If the last dimension is size 1, will encode on that
+#'     dimension. If the last dimension is not size 1, will append a new
+#'     dimension for the encoded output.
 #'   - `"multi_hot"`: Encodes each sample in the input into a single array
 #'     the same size as the vocabulary, containing a 1 for each vocabulary
 #'     term present in the sample. Treats the last dimension as the sample
 #'     dimension, if input shape is (..., sample_length), output shape will
 #'     be (..., num_tokens).
-#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of the
-#'     number of times the token at that index appeared in the sample.
+#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of
+#'     the number of times the token at that index appeared in the sample.
 #'   - `"tf_idf"`: As `"multi_hot"`, but the TF-IDF algorithm is applied to
 #'     find the value in each token slot.
 #' For `"int"` output, any shape of input and output is supported. For all
@@ -1014,12 +1071,16 @@ function(object,
 #' `"count"`, or `"tf_idf"`. If TRUE, the output will have its feature axis
 #' padded to `max_tokens` even if the number of unique tokens in the
 #' vocabulary is less than max_tokens, resulting in a tensor of shape
-#' `[batch_size, max_tokens]` regardless of vocabulary size. Defaults to `FALSE`.
+#' `[batch_size, max_tokens]` regardless of vocabulary size. Defaults to
+#' FALSE.
 #'
 #' @param sparse Boolean. Only applicable when `output_mode` is `"multi_hot"`,
-#' `"count"`, or `"tf_idf"`. If TRUE, returns a `SparseTensor` instead of a
+#' `"count"`, or `"tf_idf"`. If `TRUE`, returns a `SparseTensor` instead of a
 #' dense `Tensor`. Defaults to `FALSE`.
 #'
+#' @param encoding Optional. The text encoding to use to interpret the input
+#' strings. Defaults to `"utf-8"`.
+#'
 #' @param ... standard layer arguments.
 #'
 #' @family categorical features preprocessing layers
@@ -1036,11 +1097,12 @@ function(object,
          max_tokens = NULL,
          num_oov_indices = 1L,
          mask_token = NULL,
-         oov_token = '[UNK]',
+         oov_token = "[UNK]",
          vocabulary = NULL,
-         encoding = NULL,
+         idf_weights = NULL,
+         encoding = "utf-8",
          invert = FALSE,
-         output_mode = 'int',
+         output_mode = "int",
          sparse = FALSE,
          pad_to_max_tokens = FALSE,
          ...)
@@ -1093,6 +1155,10 @@ function(object,
 #' value(s) cannot be broadcast, an error will be raised when this layer's
 #' `build()` method is called.
 #'
+#' @param invert If `TRUE`, this layer will apply the inverse transformation
+#' to its inputs: it would turn a normalized input back into its
+#' original form.
+#'
 #' @param ... standard layer arguments.
 #'
 #' @family numerical features preprocessing layers
@@ -1104,7 +1170,9 @@ function(object,
 #'   -  <https://keras.io/api/layers/preprocessing_layers/numerical/normalization>
 #' @export
 layer_normalization <-
-function(object, axis = -1L, mean=NULL, variance=NULL, ...)
+function(object, axis = -1L,
+         mean = NULL, variance = NULL,
+         invert = FALSE, ...)
 {
   require_tf_version("2.6", "layer_normalization()")
   args <- capture_args(match.call(), list(axis = as_axis),
@@ -1141,6 +1209,31 @@ function(object, axis = -1L, mean=NULL, variance=NULL, ...)
 #' hence result in more unequal buckets, but could improve performance
 #' and resource consumption.
 #'
+#' @param output_mode Specification for the output of the layer. Defaults to
+#' `"int"`.  Values can be `"int"`, `"one_hot"`, `"multi_hot"`, or
+#' `"count"` configuring the layer as follows:
+#'
+#'   - `"int"`: Return the discretized bin indices directly.
+#'
+#'   - `"one_hot"`: Encodes each individual element in the input into an
+#'     array the same size as `num_bins`, containing a 1 at the input's bin
+#'     index. If the last dimension is size 1, will encode on that
+#'     dimension.  If the last dimension is not size 1, will append a new
+#'     dimension for the encoded output.
+#'
+#'   - `"multi_hot"`: Encodes each sample in the input into a single array
+#'     the same size as `num_bins`, containing a 1 for each bin index
+#'     index present in the sample. Treats the last dimension as the sample
+#'     dimension, if input shape is `(..., sample_length)`, output shape
+#'     will be `(..., num_tokens)`.
+#'
+#'   - `"count"`: As `"multi_hot"`, but the int array contains a count of
+#'     the number of times the bin index appeared in the sample.
+#'
+#' @param sparse Boolean. Only applicable to `"one_hot"`, `"multi_hot"`,
+#' and `"count"` output modes. If `TRUE`, returns a `SparseTensor` instead of
+#' a dense `Tensor`. Defaults to `FALSE`.
+#'
 #' @param ... standard layer arguments.
 #'
 #' @family numerical features preprocessing layers
@@ -1152,7 +1245,8 @@ function(object, axis = -1L, mean=NULL, variance=NULL, ...)
 #'   -  <https://keras.io/api/layers/preprocessing_layers/numerical/discretization>
 #' @export
 layer_discretization <-
-function(object, bin_boundaries=NULL, num_bins = NULL, epsilon = 0.01, ...)
+function(object, bin_boundaries=NULL, num_bins = NULL, epsilon = 0.01,
+         output_mode = "int", sparse = FALSE,...)
 {
   require_tf_version("2.6", "layer_discretization()")
   args <- capture_args(match.call(),
