@@ -4,9 +4,9 @@ library(dplyr, warn.conflicts = FALSE)
 library(reticulate)
 library(envir)
 
-# keras::install_keras(envname = "tf-2.6-cpu")
+use_virtualenv("r-keras")
 # tools/setup-test-envs.R
-# use_miniconda("tf-2.6-cpu", required=TRUE)
+
 
 options(tibble.print_min = 100)
 py_to_r_python.builtin.dict_items <- function(x) {
@@ -69,9 +69,8 @@ DF <-
                                            ~ setdiff(.y, c(.x, default_ignore)) %>%
                                              paste(collapse = ", "))) %>%
   filter(r_func_nm != "layer_activation_selu") %>%   # Not a real layer on keras side
-  # filter(!r_func_nm %in% c("layer_cudnn_gru", "layer_cudnn_lstm"))
-  identity()
-
+  filter(!r_func_nm %in% c("layer_cudnn_gru", "layer_cudnn_lstm")) %>%  # deprecated
+   identity()
 
 
 DF$missing_in_r_func_args[DF$r_func_nm == "layer_lambda"] %<>% sub("function", "", .)
@@ -81,6 +80,16 @@ DF %>%
   select(r_func_nm, missing_in_r_func_args) %>%
   print(n = Inf)
 
+
+# tf 2.13
+# # A tibble: 4 × 2
+#   r_func_nm                 missing_in_r_func_args
+#   <chr>                     <chr>
+# 1 layer_batch_normalization synchronized
+# 2 layer_cudnn_gru           go_backwards
+# 3 layer_cudnn_lstm          go_backwards
+# 4 layer_embedding           sparse
+
 # tf 2.11
 # # A tibble: 10 × 2
 #    r_func_nm              missing_in_r_func_args
